class UnicodeAsciiKeys(object):
    NULL = '\x00'
    START_OF_HEADING = '\x01'
    START_OF_TEXT = '\x02'
    END_OF_TEXT = '\x03'
    END_OF_TRANSMISSION = '\x04'
    ENQUIRY = '\x05'
    ACKNOWLEDGE = '\x06'
    BELL = '\x07'
    BACKSPACE = '\x08'
    CHARACTER_TABULATION = '\t'
    HORIZONTAL_TABULATION = '\t'
    TAB = '\t'
    LINE_FEED = '\n'
    NEW_LINE = '\n'
    END_OF_LINE = '\n'
    LINE_TABULATION = '\x0b'
    VERTICAL_TABULATION = '\x0b'
    FORM_FEED = '\x0c'
    CARRIAGE_RETURN = '\r'
    SHIFT_OUT = '\x0e'
    SHIFT_IN = '\x0f'
    DATA_LINK_ESCAPE = '\x10'
    DEVICE_CONTROL_ONE = '\x11'
    DEVICE_CONTROL_TWO = '\x12'
    DEVICE_CONTROL_THREE = '\x13'
    DEVICE_CONTROL_FOUR = '\x14'
    NEGATIVE_ACKNOWLEDGE = '\x15'
    SYNCHRONOUS_IDLE = '\x16'
    END_OF_TRANSMISSION_BLOCK = '\x17'
    CANCEL = '\x18'
    END_OF_MEDIUM = '\x19'
    SUBSTITUTE = '\x1a'
    ESCAPE = '\x1b'
    INFORMATION_SEPARATOR_FOUR = '\x1c'
    FILE_SEPARATOR = '\x1c'
    INFORMATION_SEPARATOR_THREE = '\x1d'
    GROUP_SEPARATOR = '\x1d'
    INFORMATION_SEPARATOR_TWO = '\x1e'
    RECORD_SEPARATOR = '\x1e'
    INFORMATION_SEPARATOR_ONE = '\x1f'
    UNIT_SEPARATOR = '\x1f'
    SPACE = ' '
    EXCLAMATION_MARK = '!'
    FACTORIAL = '!'
    BANG = '!'
    QUOTATION_MARK = '"'
    NUMBER_SIGN = '#'
    POUND_SIGN = '#'
    HASH = '#'
    CROSSHATCH = '#'
    OCTOTHORPE = '#'
    DOLLAR_SIGN = '$'
    ESCUDO = '$'
    PERCENT_SIGN = '%'
    AMPERSAND = '&'
    APOSTROPHE = "'"
    APOSTROPHE_QUOTE = "'"
    APL_QUOTE = "'"
    LEFT_PARENTHESIS = '('
    OPENING_PARENTHESIS = '('
    RIGHT_PARENTHESIS = ')'
    CLOSING_PARENTHESIS = ')'
    ASTERISK = '*'
    STAR = '*'
    PLUS_SIGN = '+'
    COMMA = ','
    DECIMAL_SEPARATOR = ','
    HYPHEN_MINUS = '-'
    HYPHEN_OR_MINUS_SIGN = '-'
    FULL_STOP = '.'
    PERIOD = '.'
    DOT = '.'
    DECIMAL_POINT = '.'
    SOLIDUS = '/'
    SLASH = '/'
    VIRGULE = '/'
    DIGIT_ZERO = '0'
    DIGIT_ONE = '1'
    DIGIT_TWO = '2'
    DIGIT_THREE = '3'
    DIGIT_FOUR = '4'
    DIGIT_FIVE = '5'
    DIGIT_SIX = '6'
    DIGIT_SEVEN = '7'
    DIGIT_EIGHT = '8'
    DIGIT_NINE = '9'
    COLON = ':'
    SEMICOLON = ';'
    LESS_THAN_SIGN = '<'
    EQUALS_SIGN = '='
    GREATER_THAN_SIGN = '>'
    QUESTION_MARK = '?'
    COMMERCIAL_AT = '@'
    AT_SIGN = '@'
    LATIN_CAPITAL_LETTER_A = 'A'
    LATIN_CAPITAL_LETTER_B = 'B'
    LATIN_CAPITAL_LETTER_C = 'C'
    LATIN_CAPITAL_LETTER_D = 'D'
    LATIN_CAPITAL_LETTER_E = 'E'
    LATIN_CAPITAL_LETTER_F = 'F'
    LATIN_CAPITAL_LETTER_G = 'G'
    LATIN_CAPITAL_LETTER_H = 'H'
    LATIN_CAPITAL_LETTER_I = 'I'
    LATIN_CAPITAL_LETTER_J = 'J'
    LATIN_CAPITAL_LETTER_K = 'K'
    LATIN_CAPITAL_LETTER_L = 'L'
    LATIN_CAPITAL_LETTER_M = 'M'
    LATIN_CAPITAL_LETTER_N = 'N'
    LATIN_CAPITAL_LETTER_O = 'O'
    LATIN_CAPITAL_LETTER_P = 'P'
    LATIN_CAPITAL_LETTER_Q = 'Q'
    LATIN_CAPITAL_LETTER_R = 'R'
    LATIN_CAPITAL_LETTER_S = 'S'
    LATIN_CAPITAL_LETTER_T = 'T'
    LATIN_CAPITAL_LETTER_U = 'U'
    LATIN_CAPITAL_LETTER_V = 'V'
    LATIN_CAPITAL_LETTER_W = 'W'
    LATIN_CAPITAL_LETTER_X = 'X'
    LATIN_CAPITAL_LETTER_Y = 'Y'
    LATIN_CAPITAL_LETTER_Z = 'Z'
    LEFT_SQUARE_BRACKET = '['
    OPENING_SQUARE_BRACKET = '['
    REVERSE_SOLIDUS = '\\'
    BACKSLASH = '\\'
    RIGHT_SQUARE_BRACKET = ']'
    CLOSING_SQUARE_BRACKET = ']'
    CIRCUMFLEX_ACCENT = '^'
    LOW_LINE = '_'
    SPACING_UNDERSCORE = '_'
    GRAVE_ACCENT = '`'
    LATIN_SMALL_LETTER_A = 'a'
    LATIN_SMALL_LETTER_B = 'b'
    LATIN_SMALL_LETTER_C = 'c'
    LATIN_SMALL_LETTER_D = 'd'
    LATIN_SMALL_LETTER_E = 'e'
    LATIN_SMALL_LETTER_F = 'f'
    LATIN_SMALL_LETTER_G = 'g'
    LATIN_SMALL_LETTER_H = 'h'
    LATIN_SMALL_LETTER_I = 'i'
    LATIN_SMALL_LETTER_J = 'j'
    LATIN_SMALL_LETTER_K = 'k'
    LATIN_SMALL_LETTER_L = 'l'
    LATIN_SMALL_LETTER_M = 'm'
    LATIN_SMALL_LETTER_N = 'n'
    LATIN_SMALL_LETTER_O = 'o'
    LATIN_SMALL_LETTER_P = 'p'
    LATIN_SMALL_LETTER_Q = 'q'
    LATIN_SMALL_LETTER_R = 'r'
    LATIN_SMALL_LETTER_S = 's'
    LATIN_SMALL_LETTER_T = 't'
    LATIN_SMALL_LETTER_U = 'u'
    LATIN_SMALL_LETTER_V = 'v'
    LATIN_SMALL_LETTER_W = 'w'
    LATIN_SMALL_LETTER_X = 'x'
    LATIN_SMALL_LETTER_Y = 'y'
    LATIN_SMALL_LETTER_Z = 'z'
    LEFT_CURLY_BRACKET = '{'
    OPENING_CURLY_BRACKET = '{'
    LEFT_BRACE = '{'
    VERTICAL_LINE = '|'
    VERTICAL_BAR = '|'
    RIGHT_CURLY_BRACKET = '}'
    CLOSING_CURLY_BRACKET = '}'
    RIGHT_BRACE = '}'
    TILDE = '~'
    DELETE = '\x7f'
